/*
 S60Bible A Bible Reader for SymbianOS
 Copyright (C) 2004 Yohanes Nugroho
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 Yohanes Nugroho (yohanes@opensource.or.id)
 Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
 */


#include "S60BibleMainContainer.h"
#include "S60Bible.hrh"
#include "util.h"
#include "CStringList.h"
#include "VerseInfo.h"
#include "Decoder.h"
#include <eikenv.h>
#include <eikspane.h>
#include <aknviewappui.h>
#include <S60Bible.rsg>
#include "log.h"

CS60BibleMainContainer* CS60BibleMainContainer::NewL(CAknView *aView, CS60BibleDocument* aDocument)
{
	CS60BibleMainContainer* self = CS60BibleMainContainer::NewLC(aView, aDocument);
	CleanupStack::Pop(self);
	return self;
}

CS60BibleMainContainer* CS60BibleMainContainer::NewLC(CAknView *aView, CS60BibleDocument* aDocument)
{
	CS60BibleMainContainer* self = new (ELeave) CS60BibleMainContainer;
	CleanupStack::PushL(self);
	self->ConstructL(aView, aDocument);
	return self;
}

const TInt KTimerTick = 10 * 1000 * 1000;

void CS60BibleMainContainer::ConstructL(CAknView *aView, CS60BibleDocument* aDocument)
{
	CreateWindowL();
	iCba= CEikButtonGroupContainer::Current();
	
	iCba->MakeVisible(ETrue);
	SetRect(aView->ClientRect());
	iView = aView;
	
	ActivateL();
	
	iFullScreen = EFalse;
	iDocument = aDocument;
	iDocument->Model()->Init();
	iDocument->Model()->SetArea(iView->ClientRect());
	iDocument->Model()->FillVerse();
	iTimer = CPeriodic::NewL(CActive::EPriorityStandard);
}

void CS60BibleMainContainer::SetBackLight(TBool aAlwaysOn)
{
	iTimer->Cancel();
	if (aAlwaysOn)
	{
		iTimer->Start(KTimerTick, KTimerTick, TCallBack(Tick, this));
	}
}

TInt CS60BibleMainContainer::Tick(TAny* /*aObject*/)
{
	User::ResetInactivityTime();
	// Return that we want the timer to continue calling us.
	return 1;
}

CS60BibleMainContainer::~CS60BibleMainContainer()
{
	iTimer->Cancel();
	delete iTimer;
}

void CS60BibleMainContainer::ToggleFullScreen()
{
	SetFullScreenL(!iFullScreen);
}

void CS60BibleMainContainer::SetFullScreenL(TBool aFullScreen)
{
	CEikonEnv *eikonEnv = CEikonEnv::Static();
	
	iFullScreen = aFullScreen;
	
	if ( iFullScreen ) {
		// Hide indicator bar and CBA:
		eikonEnv->AppUiFactory()->StatusPane()->MakeVisible(EFalse);
		iCba->MakeVisible(EFalse);
		
		// Set view to cover full screen
		SetRect(iView->ClientRect());
		iDocument->Model()->SetArea(iView->ClientRect());
		iCba->MakeVisible(ETrue);
	} else {
		// Show indicator bar and CBA:
		eikonEnv->AppUiFactory()->StatusPane()->MakeVisible(ETrue);
		iCba->MakeVisible(ETrue);		
		// Set view to cover the remaining area
		SetRect(iView->ClientRect());
		iDocument->Model()->SetArea(iView->ClientRect());
	}
	
	DrawDeferred();
}


TInt CS60BibleMainContainer::CountComponentControls() const
{
	return 0;
}



void CS60BibleMainContainer::Draw(const TRect& /*aRect*/) const
{
	CWindowGc& graphicsContext = SystemGc();
	graphicsContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	graphicsContext.SetBrushColor(iDocument->Model()->GetColor(Viewer::EVerseBackground));
	
	graphicsContext.Clear();
	TRect drawRect=Rect();
	
	if (!iDocument->Model()->IsReady()){
		HBufC* str = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_BIBLE_NOT_FOUND);
		//_LIT(msg, "Bible Not Found");
		graphicsContext.UseFont(CEikonEnv::Static()->NormalFont());
		int w = CEikonEnv::Static()->NormalFont()->TextWidthInPixels(*str);
		int ctr = (drawRect.Width()-w)/2;
		graphicsContext.DrawText(*str,  TPoint(ctr, (drawRect.Height()/2)));
		CleanupStack::PopAndDestroy(str);
		graphicsContext.DiscardFont();
		return;
	}
	
	const CFont *font = iDocument->Model()->GetFont();
	char *s = NULL;
	if (iDocument->Model()->UseLongBookName()) {
		s = iDocument->Model()->BookName();
	} else {
		s = iDocument->Model()->SimpleBookName();
	}
	TBuf<256> stat;
	iDocument->Model()->MyStringToDes(s, stat);
	
	User::Free(s);
	
	VerseInfo *v = iDocument->Model()->GetVerseHead();
	
	int start_chapter, start_verse, start_line;
	start_chapter = iDocument->Model()->GetStartChapter();
	start_verse = iDocument->Model()->GetStartVerse();
	start_line = iDocument->Model()->GetStartLine();
	
	
	while (v) {
		if (v->verse==start_verse && v->chapter==start_chapter) {
			break;
		}
		v = v->next;
	}
 

	int EOrientationNormal = 0;
	int EOrientationLandscape90 = 1;
	int EOrientationLandscape270 = 2;
	int iOrientation = EOrientationLandscape270;
	iOrientation=0;
	TBool up = iOrientation==EOrientationLandscape90;	
	
	
	TBuf<4> chapnum;
	chapnum.Zero();
	chapnum.AppendNum(start_chapter);
	
	graphicsContext.UseFont(font);
	
	TPoint start;
	if (iOrientation==EOrientationNormal) {		
	 start.SetXY(15 , 20 + font->HeightInPixels());
	} else {
		if (up) {
	 		start.SetXY(20 + font->HeightInPixels(), drawRect.Height()-15);	
		} else {
			start.SetXY(20 + font->HeightInPixels(), 15);	
		}
	}
	graphicsContext.UseFont(font);
	
	TRect frame;
	
	if (iOrientation==EOrientationNormal) {	
		frame.SetRect(0, 0, drawRect.Width(), 20 + font->HeightInPixels());
	} else {
		if (up) {
			frame.SetRect(0, 0, 20 + font->HeightInPixels(), drawRect.Height());
		} else {
			frame.SetRect(drawRect.Width()-20 - font->HeightInPixels(), 0, drawRect.Width(), drawRect.Height());
			
		}
	}
	graphicsContext.SetBrushStyle(CGraphicsContext::ESolidBrush);
	graphicsContext.SetBrushColor(iDocument->Model()->GetColor(Viewer::EHeaderBG));
	graphicsContext.SetPenColor(iDocument->Model()->GetColor(Viewer::EHeaderFrame));
	graphicsContext.DrawRect(frame);
	frame.Shrink(1,1);
	graphicsContext.DrawRect(frame);
	graphicsContext.SetBrushStyle(CGraphicsContext::ENullBrush);
	graphicsContext.SetPenColor(iDocument->Model()->GetColor(Viewer::EHeaderText));
	
	int chap_draw_x = font->TextWidthInPixels(stat) + font->MaxCharWidthInPixels();
	
	if (iOrientation==EOrientationNormal) {	
		graphicsContext.DrawText(stat, TPoint(10, font->HeightInPixels()+5));
		graphicsContext.DrawText(chapnum, TPoint(chap_draw_x, font->HeightInPixels()+5));
	} else {
		if (up) {
			graphicsContext.DrawTextVertical(stat, TPoint(font->HeightInPixels()+5,
				drawRect.Height()-10), up);
			graphicsContext.DrawTextVertical(stat, TPoint(font->HeightInPixels()+5,
				drawRect.Height()-chap_draw_x), up);
			
		} else {
			graphicsContext.DrawTextVertical(stat, TPoint(drawRect.Width()-(font->HeightInPixels()+5),
				10), up);
			graphicsContext.DrawTextVertical(stat, TPoint(drawRect.Width()-(font->HeightInPixels()+5),
				chap_draw_x), up);
				
		}	
	}
	if (iOrientation==EOrientationNormal) {	
		start.iY += font->HeightInPixels(); 
	} else {
		if (up) {
			start.iX += font->HeightInPixels();
		} else {
			start.iX += font->HeightInPixels();			
			start.iX = drawRect.Width()-start.iX;
		}
	}
	TBuf<4> strversenum;
	TRgb textColor = iDocument->Model()->GetColor(Viewer::EVerseText);
	TRgb numberColor = iDocument->Model()->GetColor(Viewer::EVerseNumber);
	graphicsContext.SetPenColor(textColor);
	//Normal, Landscape90, Landscape270
	int starttext = up?drawRect.Height():2;
	int iidx = start_line;
	if (!iDocument->Model()->IsRToL()){ 
		while (v) {
			graphicsContext.SetPenColor(textColor);
			int inlimit = 1;
			if (iOrientation==EOrientationNormal) {				
					inlimit = start.iY<drawRect.Height();
			} else {
					if (up) {
						inlimit = start.iX<drawRect.Width();
					} else {
						inlimit = start.iX>0;
					}
			}
			/*Draw Verses*/
			while (iidx<v->lines->Count() && inlimit) {

				if (iidx == v->verseline) {

					strversenum.Zero();
					strversenum.AppendNum(v->verse);
					int fw = font->TextWidthInPixels(strversenum);
					int sw = fw+5;
					graphicsContext.SetPenColor(numberColor);
					if (iOrientation==EOrientationNormal) {
						if (fw==0) {
							graphicsContext.DiscardFont();
							graphicsContext.UseFont(CEikonEnv::Static()->NormalFont());
							sw = CEikonEnv::Static()->NormalFont()->TextWidthInPixels(strversenum)+5;
						}							
						graphicsContext.DrawText(strversenum,  TPoint(2, start.iY));
						if (fw==0) {
							graphicsContext.DiscardFont();
							graphicsContext.UseFont(font);
						}						
						graphicsContext.SetPenColor(textColor);	

						graphicsContext.DrawText(v->lines->At(iidx), TPoint(sw, start.iY));

					} else {

						int yp = up?drawRect.Height()-2:2;
						graphicsContext.DrawTextVertical(strversenum,  TPoint(start.iX, yp), up);
						graphicsContext.SetPenColor(textColor);
						yp = up?drawRect.Height()-sw:sw;
						graphicsContext.DrawTextVertical(v->lines->At(iidx), TPoint(start.iX, yp), up);
					}
				} else {
					
					if (v->lineformat->At(iidx)==Decoder::versTextType) {
						graphicsContext.SetPenColor(textColor);
					} else {
						graphicsContext.SetPenColor(iDocument->Model()->GetColor(Viewer::EHeaderText));
					}
					if (iOrientation==EOrientationNormal) {
						graphicsContext.DrawText((TPtrC16)v->lines->At(iidx), TPoint(2, start.iY));
					} else {
						graphicsContext.DrawTextVertical((TPtrC16)v->lines->At(iidx), TPoint(start.iX, starttext), up);					
					}
				}
				if (iOrientation==EOrientationNormal) {				
					start.iY += font->HeightInPixels();
					inlimit = start.iY<drawRect.Height();
				} else {
						if (up) {
							start.iX += font->HeightInPixels();
							inlimit = start.iX<drawRect.Width();
						} else {
							start.iX -= font->HeightInPixels();
							inlimit = start.iX>0;
						}
				}
				iidx++;
			}
			iidx = 0;
			v = v->next;
		}
	} else { /*Right to left*/
		while (v) {
			graphicsContext.SetPenColor(textColor);
			int inlimit = 1;
				if (iOrientation==EOrientationNormal) {				
					inlimit = start.iY<drawRect.Height();
				} else {
						if (up) {
							inlimit = start.iX<drawRect.Width();
						} else {
							inlimit = start.iX>0;
						}
				}
			/*Draw Verses RTOL*/
			while (iidx<v->lines->Count() && inlimit) {
				if (iidx == v->verseline) {
					strversenum.Zero();
					strversenum.AppendNum(v->verse);
					graphicsContext.SetPenColor(numberColor);
					int sw = font->TextWidthInPixels(strversenum);
					int wl = font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));	
					if (iOrientation==EOrientationNormal) {
						graphicsContext.DrawText(strversenum,  TPoint(drawRect.Width()-sw, start.iY));
						graphicsContext.SetPenColor(textColor);
						graphicsContext.DrawText(v->lines->At(iidx), TPoint(drawRect.Width()-sw-wl-5, start.iY));
					} else {
						int yp = up?sw:drawRect.Height()-sw;
						graphicsContext.DrawTextVertical(strversenum,  TPoint(start.iX, yp), up);
						graphicsContext.SetPenColor(textColor);
						yp = up?sw+wl+5:drawRect.Height()-(sw+wl+5);
						graphicsContext.DrawTextVertical(v->lines->At(iidx), TPoint(start.iX, yp), up);
					}
				} else {
					if (iOrientation==EOrientationNormal) {
						int pos = drawRect.Width() - font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));
						graphicsContext.DrawText((TPtrC16)v->lines->At(iidx), TPoint(pos, start.iY));
					} else {
						if (up) {
							int pos = font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));
							graphicsContext.DrawTextVertical((TPtrC16)v->lines->At(iidx), TPoint(start.iX, pos), up);	
						} else {
							int pos = drawRect.Height() - font->TextWidthInPixels((TPtrC16)v->lines->At(iidx));
							graphicsContext.DrawTextVertical((TPtrC16)v->lines->At(iidx), TPoint(start.iX, pos), up);	
							
						}
					}

				}

				if (iOrientation==EOrientationNormal) {				
					start.iY += font->HeightInPixels();
					inlimit = start.iY<drawRect.Height();
				} else {
						if (up) {
							start.iX += font->HeightInPixels();
							inlimit = start.iX<drawRect.Width();
						} else {
							start.iX -= font->HeightInPixels();
							inlimit = start.iX>0;
						}
				}
				iidx++;
			}
			iidx = 0;
			v = v->next;
		}
	}
	graphicsContext.DiscardFont();
}

void CS60BibleMainContainer::ExecuteCommand(int idx)
{
	/*adjust these according to s60bible.rss*/
	int commands[] =
{
	0,
	KPrevLine,
	KNextLine,
	KPrevPage,
	KNextPage,
	KPrevVerse,
	KNextVerse,
	KPrevChapter,
	KNextChapter,
	KPrevBook,
	KNextBook,
	KGotoBookmark,
	KShowHistory,
	KBookList,
	KGotoTop,
	KGotoBottom,
	KFullscreen
};
iView->HandleCommandL(commands[idx]);
}

#ifdef __SERIES60_30__
void CS60BibleMainContainer::HandleResourceChange(TInt aType)
{
	CCoeControl::HandleResourceChange(aType);
    if  ( aType == KEikDynamicLayoutVariantSwitch ) {
    	TRect rect;
    	if (iFullScreen) {
    		AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EApplicationWindow , rect);
    	} else {
    		AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EMainPane, rect);
    	}
		iDocument->Model()->SetArea(rect);
		iDocument->Model()->ClearVerseList();
		iDocument->Model()->FillVerse();		
		SetRect(rect);	
    }
}
#endif

TKeyResponse CS60BibleMainContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	
	if (aType != EEventKey){
		return CCoeControl::OfferKeyEventL(aKeyEvent, aType);
	}
	
	/*some special keys*/
	if (aKeyEvent.iScanCode==EStdKeyUpArrow){
		int x = iDocument->Model()->GetBinding(10);
		if (x!=0) {
			ExecuteCommand(x);
		} else {
			iDocument->Model()->PageUp();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	
	if (aKeyEvent.iScanCode==EStdKeyDownArrow) {
		int x = iDocument->Model()->GetBinding(11);
		if (x) {
			ExecuteCommand(x);
		} else {
			iDocument->Model()->PageDown();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	
	if (aKeyEvent.iScanCode==EStdKeyLeftArrow) {
		int x = iDocument->Model()->GetBinding(12);
		if (x) {
			ExecuteCommand(x);
		} else {
			iDocument->Model()->ScrollUp(1);
			iDocument->Model()->UpdateBuffer();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	if (aKeyEvent.iScanCode==EStdKeyRightArrow) {
		int x = iDocument->Model()->GetBinding(13);
		if (x) {
			ExecuteCommand(x);
		} else {
			
			iDocument->Model()->ScrollDown(1);
			iDocument->Model()->UpdateBuffer();
			DrawDeferred();
		}
		return EKeyWasConsumed;
	}
	
	if (aKeyEvent.iScanCode == EStdKeyDevice3)
	{
		int x = iDocument->Model()->GetBinding(14);
		if (x) {
			ExecuteCommand(x);
			return EKeyWasConsumed;
		}
	}
	
	if (aKeyEvent.iCode>='0' && aKeyEvent.iCode<='9') {
		int x = iDocument->Model()->GetBinding(aKeyEvent.iCode-'0');
		if (x) {
			ExecuteCommand(x);
			return EKeyWasConsumed;
		}
	}
	
	/*some default mapping*/
	if (aKeyEvent.iCode=='9') {
		iView->HandleCommandL(KFullscreen);
		return EKeyWasConsumed;
	}
	if (aKeyEvent.iScanCode == EStdKeyDevice3)
	{
		iView->HandleCommandL(KBookList);
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
	
}

TCoeInputCapabilities CS60BibleMainContainer::InputCapabilities() const
{
	// this class does not implement any 'unusual' input capabilities
	return TCoeInputCapabilities::ENone;
}


CCoeControl* CS60BibleMainContainer::ComponentControl(TInt /*aIndex*/) const
{
	return NULL;
}

